@@ -32,7 +32,6 @@
 #include <config.h>
 #endif
 
-#include <string.h>
 #include <gtk/gtk.h>
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -44,9 +43,20 @@
 #include "gedit-view.h"
 #include "gedit-file.h"
 #include "gedit-print.h"
+#include "gedit-prefs.h"
 #include "dialogs/gedit-dialogs.h"
 #include "dialogs/gedit-preferences-dialog.h"
 
+#define TO_BE_IMPLEMENTED	{ GtkWidget *message_dlg; \
+				  message_dlg = gtk_message_dialog_new (	\
+			                          GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),	\
+			                          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,	\
+			                          GTK_MESSAGE_INFO,	\
+			                          GTK_BUTTONS_OK,	\
+                                   		  _("Not yet implemented."));	\
+				  gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK); \
+	                          gtk_dialog_run (GTK_DIALOG (message_dlg));	\
+  	                          gtk_widget_destroy (message_dlg); }
 
 void 
 gedit_cmd_file_new (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
@@ -79,7 +89,7 @@ gedit_cmd_file_save (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 	if (active_child == NULL)
 		return;
 	
-	gedit_file_save (active_child, TRUE);
+	gedit_file_save (active_child);
 }
 
 void 
@@ -212,9 +222,7 @@ gedit_cmd_edit_undo (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 
 	gedit_document_undo (active_document);
 
-	gedit_view_scroll_to_cursor (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	/* TODO: Move the cursor */
 }
 
 void 
@@ -231,9 +239,7 @@ gedit_cmd_edit_redo (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 
 	gedit_document_redo (active_document);
 
-	gedit_view_scroll_to_cursor (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	/* TODO: Move the cursor */
 }
 
 void 
@@ -245,8 +251,6 @@ gedit_cmd_edit_cut (BonoboUIComponent *uic, gpointer user_data, const gchar* ver
 	g_return_if_fail (active_view);
 	
 	gedit_view_cut_clipboard (active_view); 
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
 }
 
 void 
@@ -257,9 +261,7 @@ gedit_cmd_edit_copy (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
 	
-	gedit_view_copy_clipboard (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	gedit_view_copy_clipboard (active_view); 
 }
 
 void 
@@ -271,8 +273,6 @@ gedit_cmd_edit_paste (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 	g_return_if_fail (active_view);
 	
 	gedit_view_paste_clipboard (active_view); 
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
 }
 
 void 
@@ -283,26 +283,18 @@ gedit_cmd_edit_clear (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
 	
-	gedit_view_delete_selection (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	gedit_view_delete_selection (active_view); 
 }
 
 void
 gedit_cmd_edit_select_all (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	GeditDocument* active_doc;
 	GeditView* active_view;
 
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
-
-	active_doc = gedit_get_active_document ();
-	g_return_if_fail (active_doc);
-
-	gedit_document_set_selection (active_doc, 0, -1); 
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	
+	gedit_view_select_all (active_view); 
 }
 
 void 
@@ -335,31 +327,16 @@ gedit_cmd_search_find_again (BonoboUIComponent *uic, gpointer user_data, const g
 	last_searched_text = gedit_document_get_last_searched_text (doc);
 	if (last_searched_text != NULL)
 	{
-		gpointer data;
-		gboolean was_wrap_around;
-		gboolean found;
-
-		data = g_object_get_qdata (G_OBJECT (doc), gedit_was_wrap_around_quark ());
-		if (data == NULL)
-			was_wrap_around = TRUE;
-		else
-			was_wrap_around = GPOINTER_TO_BOOLEAN (data);
-
-		found = gedit_document_find_again (doc, TRUE);
-
-		if (!found && was_wrap_around)
-			found = gedit_document_find_again (doc, FALSE);
-
-		if (!found)
+		if (!gedit_document_find_again (doc))
 		{	
 			GtkWidget *message_dlg;
 
 			message_dlg = gtk_message_dialog_new (
-				GTK_WINDOW (gedit_get_active_window ()),
+				GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
 				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
-				_("The text \"%s\" was not found."), last_searched_text);
+				_("The string \"%s\" has not been found."), last_searched_text);
 
 			gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
 
@@ -371,10 +348,12 @@ gedit_cmd_search_find_again (BonoboUIComponent *uic, gpointer user_data, const g
 		else
 			gedit_view_scroll_to_cursor (active_view);
 
-		g_free (last_searched_text);
 	}
 	else
+	{
+		g_free (last_searched_text);
 		gedit_dialog_find ();
+	}
 }
 
 
@@ -397,78 +376,96 @@ gedit_cmd_search_goto_line (BonoboUIComponent *uic, gpointer user_data, const gc
 void
 gedit_cmd_settings_preferences (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	static GtkWidget *dlg = NULL;
+	GtkWidget *dlg;
+	gint ret;
 
 	gedit_debug (DEBUG_COMMANDS, "");
 
-	if (dlg != NULL)
-	{
-		gtk_window_present (GTK_WINDOW (dlg));
-		gtk_window_set_transient_for (GTK_WINDOW (dlg),	
-					      GTK_WINDOW (gedit_get_active_window ()));
+	dlg = gedit_preferences_dialog_new (
+			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))));
 
-		return;
-	}
+	do
+	{
+		ret = gtk_dialog_run (GTK_DIALOG (dlg));
+
+		switch (ret)
+		{
+			case GTK_RESPONSE_OK:
+				if (gedit_preferences_dialog_update_settings (GEDIT_PREFERENCES_DIALOG (dlg)))
+				{
+					gedit_mdi_update_ui_according_to_preferences (gedit_mdi);
+					gedit_prefs_save_settings ();
+
+					gtk_widget_hide (dlg);
+				}
+				else
+				{
+					GtkWidget *message_dlg;
+
+					message_dlg = gtk_message_dialog_new (
+						GTK_WINDOW (bonobo_mdi_get_active_window (
+								BONOBO_MDI (gedit_mdi))),
+						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						GTK_MESSAGE_ERROR,
+						GTK_BUTTONS_OK,
+						_("Impossible to update gedit settings."));
+
+					gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), 
+							GTK_RESPONSE_OK);
+
+					gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
+
+					gtk_dialog_run (GTK_DIALOG (message_dlg));
+  					gtk_widget_destroy (message_dlg);
+				}
+
+				break;
+								
+			case GTK_RESPONSE_HELP:
+				/* FIXME */
+				break;
+
+			default:
+				gtk_widget_hide (dlg);
+		}	
 		
-	dlg = gedit_preferences_dialog_new (GTK_WINDOW (gedit_get_active_window ()));
+	} while (GTK_WIDGET_VISIBLE (dlg));
 
-	g_signal_connect (G_OBJECT (dlg), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &dlg);
-	
-	gtk_widget_show (dlg);
-}
-
-void
-gedit_cmd_documents_move_to_new_window (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
-{
-	GeditView *view;
-
-	gedit_debug (DEBUG_COMMANDS, "");
-
-	view = gedit_get_active_view ();
-	g_return_if_fail (view != NULL);
-
-	bonobo_mdi_move_view_to_new_window (BONOBO_MDI (gedit_mdi), GTK_WIDGET (view));
-}
-
-void 
-gedit_cmd_help_contents (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
-{
-	GError *error = NULL;
-
-	gedit_debug (DEBUG_COMMANDS, "");
-
-	gnome_help_display ("gedit.xml", NULL, &error);
-	
-	if (error != NULL)
-	{
-		g_warning (error->message);
-
-		g_error_free (error);
-	}
+	gtk_widget_destroy (dlg);
 }
 
 void 
 gedit_cmd_help_about (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
+	/* TODO: add logo -- Paolo */
+	/* FIXME: set window position at the center of the active window -- Paolo */
+
 	static GtkWidget *about = NULL;
-	GdkPixbuf* pixbuf = NULL;
+	GdkPixbuf* pixbuf;
 	
 	gchar *authors[] = {
-		"Paolo Maggi <maggi@athena.polito.it>",
-		"Chema Celorio <chema@ximian.com>", 
+		"", "", 
+		/*
+		"", "", "", "",
 		"James Willcox <jwillcox@cs.indiana.edu>",
-		"Federico Mena Quintero <federico@ximian.com>",
+		"Jason Leach <jasonleach@usa.net>",
+		N_("Thanks to:"),
+		"",
+		*/
+		"Paolo Maggi <maggi@athena.polito.it>",
+		"Chema Celorio <chema@celorio.com>", 
 		NULL
 	};
 	
 	gchar *documenters[] = {
-		"Sun GNOME Documentation Team <gdocteam@sun.com>",
 		"Eric Baudais <baudais@okstate.edu>",
 		NULL
 	};
 	
-	gchar *translator_credits = _("translator_credits");
+	/* Translator credits */
+	/*
+	gchar *translator_credits = _("");
+	*/
 
 	gedit_debug (DEBUG_COMMANDS, "");
 
@@ -479,42 +476,29 @@ gedit_cmd_help_about (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 		return;
 	}
 	
-	pixbuf = gdk_pixbuf_new_from_file ( GNOME_ICONDIR "/gedit-logo.png", NULL);
-	if (pixbuf != NULL)
-	{
-		GdkPixbuf* temp_pixbuf = NULL;
-
-		temp_pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
-					 gdk_pixbuf_get_width (pixbuf),
-					 gdk_pixbuf_get_height (pixbuf),
-					 GDK_INTERP_HYPER);
-		g_object_unref (pixbuf);
-
-		pixbuf = temp_pixbuf;
-	}
-
 	about = gnome_about_new (_("gedit"), VERSION,
-				"Copyright \xc2\xa9 1998-2000 Evan Lawrence, Alex Robert\n"
-				"Copyright \xc2\xa9 2000-2002 Chema Celorio, Paolo Maggi",
+				"(C) 1998-2000 Evan Lawrence and Alex Robert\n"
+				"(C) 2000-2002 Chema Celorio and Paolo Maggi",
 				_("gedit is a small and lightweight text editor for Gnome"),
 				(const char **)authors,
 				(const char **)documenters,
-				strcmp (translator_credits, "translator_credits") != 0 ? 
-					(const char *)translator_credits : NULL,
-				pixbuf);
+				NULL, /*(const char *)translator_credits,*/
+				NULL);
 
-	gtk_window_set_transient_for (GTK_WINDOW (about),
-			GTK_WINDOW (gedit_get_active_window ()));
-
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);
+	pixbuf = gdk_pixbuf_new_from_file ( GNOME_ICONDIR "/gedit-logo.png", NULL);
 
 	if (pixbuf != NULL)
+	{
+		g_object_set (G_OBJECT (about), "background", pixbuf,
+			"background_start_opacity", 0.4,
+			"background_end_opacity", 0.1, NULL);
+
 		g_object_unref (pixbuf);
+	}
 	
-	g_signal_connect (G_OBJECT (about), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &about);
+	gtk_signal_connect (GTK_OBJECT (about), "destroy",
+			   GTK_SIGNAL_FUNC (gtk_widget_destroyed), &about);
 	
 	gtk_widget_show (about);
 }
 
-
