@@ -33,7 +33,9 @@
 
 #include <unistd.h>  
 #include <stdio.h>
+#include <string.h>
 
+#include "gedit-prefs.h"
 #include "gnome-vfs-helpers.h"
 #include "gedit-document.h"
 #include "gedit-debug.h"
@@ -49,6 +51,8 @@ struct _GeditDocumentPrivate
 	gint 	untitled_number;	
 
 	gchar*  last_searched_text;
+	gchar*  last_replace_text;
+	gboolean  last_search_was_case_sensitive;
 	
 	gboolean readonly;
 
@@ -80,7 +84,7 @@ static void gedit_document_real_readonly_changed	(GeditDocument *document,
 							 gboolean readonly);
 
 static gboolean	gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, 
-					     GError **error);
+					     gboolean create_backup_copy, GError **error);
 static void gedit_document_set_uri (GeditDocument* doc, const gchar* uri);
 
 static void gedit_document_can_undo_handler (GeditUndoManager* um, gboolean can_undo, 
@@ -281,6 +285,9 @@ gedit_document_finalize (GObject *object)
 	if (document->priv->last_searched_text)
 		g_free (document->priv->last_searched_text);
 
+	if (document->priv->last_replace_text)
+		g_free (document->priv->last_replace_text);
+
 	g_object_unref (G_OBJECT (document->priv->undo_manager));
 	
 	g_free (document->priv);
@@ -354,6 +361,9 @@ gedit_document_new_with_uri (const gchar *uri, GError **error)
 void 		
 gedit_document_set_readonly (GeditDocument *document, gboolean readonly)
 {
+	GtkTextIter start;
+	GtkTextIter end;
+
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_if_fail (document != NULL);
@@ -362,6 +372,22 @@ gedit_document_set_readonly (GeditDocument *document, gboolean readonly)
 	if (document->priv->readonly == readonly) 
 		return;
 
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (document), &start, 0);
+      	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (document), &end);
+
+	if(readonly) 
+	{
+		gtk_text_buffer_apply_tag_by_name (GTK_TEXT_BUFFER (document), 
+					NOT_EDITABLE_TAG_NAME, &start, &end);
+	}
+	else
+	{
+		gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER (document), 
+					NOT_EDITABLE_TAG_NAME, &start, &end);
+	}
+
+	document->priv->readonly = readonly;
+
 	g_signal_emit (G_OBJECT (document),
                        document_signals[READONLY_CHANGED],
                        0,
@@ -414,29 +440,9 @@ gedit_document_real_saved (GeditDocument *document)
 static void 
 gedit_document_real_readonly_changed (GeditDocument *document, gboolean readonly)
 {
-	GtkTextIter start;
-	GtkTextIter end;
-	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_if_fail (document != NULL);
-	g_return_if_fail (document->priv != NULL);
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (document), &start, 0);
-      	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (document), &end);
-
-	if(readonly) 
-	{
-		gtk_text_buffer_apply_tag_by_name (GTK_TEXT_BUFFER (document), 
-					NOT_EDITABLE_TAG_NAME, &start, &end);
-	}
-	else
-	{
-		gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER (document), 
-					NOT_EDITABLE_TAG_NAME, &start, &end);
-	}
-
-	document->priv->readonly = readonly;
 }
 
 gchar*
@@ -458,7 +464,7 @@ gedit_document_get_short_name (const GeditDocument* doc)
 	if (doc->priv->uri == NULL)
 		return g_strdup_printf (_("%s %d"), _("Untitled"), doc->priv->untitled_number);
 	else
-		gnome_vfs_x_uri_get_basename (doc->priv->uri);
+		return gnome_vfs_x_uri_get_basename (doc->priv->uri);
 }
 
 GQuark 
@@ -486,7 +492,7 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 {
 	char* file_contents;
 	GnomeVFSResult res;
-   	int file_size;
+   	gsize file_size;
 	GtkTextIter iter, end;
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
@@ -507,10 +513,35 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 	{
 		if (!g_utf8_validate (file_contents, file_size, NULL))
 		{
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_WRONG_FORMAT,
-				_("Invalid UTF-8 data encountered reading file"));
+			/* The file contains invalid UTF8 data */
+			/* Try to convert it to UTF-8 from currence locale */
+			GError *conv_error = NULL;
+			gchar* converted_file_contents = NULL;
+			gsize bytes_written;
+			
+			converted_file_contents = g_locale_to_utf8 (file_contents, file_size,
+					NULL, &bytes_written, &conv_error); 
+			
 			g_free (file_contents);
-			return FALSE;
+
+			if ((conv_error != NULL) || 
+			    !g_utf8_validate (converted_file_contents, bytes_written, NULL))		
+			{
+				/* Coversion failed */	
+				if (conv_error != NULL)
+					g_error_free (conv_error);
+
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_WRONG_FORMAT,
+				_("Invalid UTF-8 data encountered reading file"));
+				
+				if (converted_file_contents != NULL)
+					g_free (converted_file_contents);
+				
+				return FALSE;
+			}
+
+			file_contents = converted_file_contents;
+			file_size = bytes_written;
 		}
 
 		gedit_undo_manager_begin_not_undoable_action (doc->priv->undo_manager);
@@ -678,7 +709,8 @@ gedit_document_save (GeditDocument* doc, GError **error)
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (!doc->priv->readonly, FALSE);
 
-	return gedit_document_save_as_real (doc, doc->priv->uri, error);
+	return gedit_document_save_as_real (doc, doc->priv->uri, 
+			gedit_settings->create_backup_copy, error);
 }
 
 gboolean	
@@ -689,9 +721,8 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
-	g_return_val_if_fail (!doc->priv->readonly, FALSE);
 
-	if (gedit_document_save_as_real (doc, uri, error))
+	if (gedit_document_save_as_real (doc, uri, FALSE, error))
 	{
 		gedit_document_set_uri (doc, uri);
 		gedit_document_set_readonly (doc, FALSE);
@@ -702,19 +733,21 @@ gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
 }
 
 static gboolean	
-gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, GError **error)
+gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
+	       gboolean create_backup_copy, GError **error)
 {
-	/* TODO: backup file? */
 	FILE* file;
 	gchar* chars;
 	gchar* fname;
-
+	gchar* bak_fname = NULL;
+	gboolean have_backup = FALSE;
+	gboolean res = FALSE;
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
-	g_return_val_if_fail (!doc->priv->readonly, FALSE);
 
 	if (!gedit_utils_uri_has_file_scheme (uri))
 	{
@@ -725,35 +758,93 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, GError **erro
 	
 	fname = gnome_vfs_x_format_uri_for_display (uri);
 
+	if (create_backup_copy && (fname != NULL))
+	{
+		bak_fname = g_strconcat (fname, gedit_settings->backup_extension, NULL);
+		
+		if ((bak_fname == NULL) || (rename (fname, bak_fname) != 0))
+    		{
+      			if (errno != ENOENT)
+			{
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
+					"Cannot create the backup copy of the file.\n"
+					"The file has not been saved.");
+		
+				if (fname != NULL) 
+					g_free (fname);
+
+				if (bak_fname != NULL) 
+					g_free (bak_fname);
+
+				return FALSE;
+
+			}
+    		}
+  		else
+    			have_backup = TRUE;
+	}
+
 	if ((fname == NULL) || ((file = fopen (fname, "w")) == NULL))
 	{
 		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
 			"Make sure that the path you provided exists,\n"
 			"and that you have the appropriate write permissions.");
-		
-		if (fname != NULL) 
-			g_free (fname);
-		
-		return FALSE;
-	}
+						
+		res = FALSE;
 
-	g_free (fname);
+		goto finally;
+	}
 
       	chars = gedit_document_get_buffer (doc);
 
+	if (gedit_settings->save_encoding == GEDIT_SAVE_CURRENT_LOCALE_WHEN_POSSIBLE)
+	{
+		GError *conv_error = NULL;
+		gchar* converted_file_contents = NULL;
+
+		converted_file_contents = g_locale_from_utf8 (chars, -1, NULL, NULL, 
+				&conv_error);
+
+		if (conv_error != NULL)
+		{
+			/* Conversion error */
+			g_error_free (conv_error);
+		}
+		else
+		{
+			g_free (chars);
+			chars = converted_file_contents;
+		}
+	}
+	
 	if (fputs (chars, file) == EOF || fclose (file) == EOF)
 	{
 		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno, g_strerror (errno));
 		g_free (chars);
 
-		return FALSE;
+		res = FALSE;
+
+		goto finally;
 	}
 
 	g_free (chars);
 
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);	  
 
-	return TRUE;
+	res = TRUE;
+	
+finally:
+	if (!res && have_backup)
+		/* FIXME: should the errors be managed ? - Paolo */
+      		rename (bak_fname, fname);
+	
+	if (fname != NULL) 
+		g_free (fname);
+
+	if (bak_fname != NULL) 
+		g_free (bak_fname);
+
+  	return res;
 }
 
 gchar* 
@@ -1011,6 +1102,25 @@ gedit_document_end_not_undoable_action	(GeditDocument *doc)
 	gedit_undo_manager_end_not_undoable_action (doc->priv->undo_manager);
 }
 
+void		
+gedit_document_begin_user_action (GeditDocument *doc)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (doc));
+}	
+
+void		
+gedit_document_end_user_action (GeditDocument *doc)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (doc));
+}
 
 
 static void
@@ -1062,13 +1172,25 @@ gedit_document_goto_line (GeditDocument* doc, guint line)
 
 gchar* 
 gedit_document_get_last_searched_text (GeditDocument* doc)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+	g_return_val_if_fail (doc->priv != NULL, FALSE);
+	
+	return doc->priv->last_searched_text != NULL ? 
+		g_strdup (doc->priv->last_searched_text) : NULL;	
+}
+
+gchar*
+gedit_document_get_last_replace_text (GeditDocument* doc)
 {
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 
-	return doc->priv->last_searched_text != NULL ? 
-		g_strdup (doc->priv->last_searched_text) : NULL;	
-}	
+	return doc->priv->last_replace_text != NULL ? 
+		g_strdup (doc->priv->last_replace_text) : NULL;	
+}
 
 gboolean
 gedit_document_find (GeditDocument* doc, const gchar* str, 
@@ -1076,6 +1198,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 {
 	GtkTextIter iter;
 	gboolean found = FALSE;
+	GtkTextSearchFlags search_flags;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
@@ -1083,8 +1206,16 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (str != NULL, FALSE);
 
-	/* FIXME: write support for case_sensitive */
+	search_flags = GTK_TEXT_SEARCH_VISIBLE_ONLY | GTK_TEXT_SEARCH_TEXT_ONLY;
 	
+	/* FIXME: enable support for case sensitive -- Paolo */
+	/*
+	if (!case_sensitive)
+	{
+		search_flags = search_flags | GTK_TEXT_SEARCH_CASE_INSENSITIVE;
+	}
+	*/
+
 	if (from_cursor)
 	{
 		GtkTextIter sel_bound;
@@ -1108,11 +1239,9 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
     	{
       		GtkTextIter match_start, match_end;
 
-          	found = gtk_text_iter_forward_search (&iter, str,
-                                               GTK_TEXT_SEARCH_VISIBLE_ONLY |
-                                               GTK_TEXT_SEARCH_TEXT_ONLY,
-                                               &match_start, &match_end,
-                                               NULL);
+          	found = gtk_text_iter_forward_search (&iter, str, search_flags,
+                                        	&match_start, &match_end,
+                                               	NULL);
 		if (found)
 		{
 			gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc),
@@ -1126,6 +1255,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 			g_free (doc->priv->last_searched_text);
 
 		doc->priv->last_searched_text = g_strdup (str);
+		doc->priv->last_search_was_case_sensitive = case_sensitive;
 	}
 
 	return found;
@@ -1147,8 +1277,126 @@ gedit_document_find_again (GeditDocument* doc)
 	if (last_searched_text == NULL)
 		return FALSE;
 	
-	found = gedit_document_find (doc, last_searched_text, TRUE, /* FIXME */FALSE);
+	found = gedit_document_find (doc, last_searched_text, TRUE, 
+				doc->priv->last_search_was_case_sensitive);
 	g_free (last_searched_text);
 
 	return found;
 }
+
+gchar*
+gedit_document_get_selected_text (GeditDocument *doc)
+{
+	GtkTextIter iter;
+	GtkTextIter sel_bound;
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &iter,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "insert"));
+		
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &sel_bound,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "selection_bound"));
+	gtk_text_iter_order (&sel_bound, &iter);	
+
+	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &iter, &sel_bound, TRUE);
+}
+
+gboolean
+gedit_document_has_selected_text (GeditDocument *doc)
+{
+	GtkTextIter iter;
+	GtkTextIter sel_bound;
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &iter,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "insert"));
+		
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &sel_bound,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "selection_bound"));
+
+	return !gtk_text_iter_equal (&sel_bound, &iter);	
+}
+
+void
+gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
+{
+	GtkTextIter iter;
+	GtkTextIter sel_bound;
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (replace != NULL);
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &iter,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "insert"));
+		
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &sel_bound,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "selection_bound"));
+		
+	gtk_text_iter_order (&sel_bound, &iter);		
+
+	gedit_document_begin_user_action (doc);
+
+	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc),
+				&iter,
+				&sel_bound);
+
+	gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc),
+				&iter,
+				replace, strlen (replace));
+
+	if (doc->priv->last_replace_text)
+		g_free (doc->priv->last_replace_text);
+
+	doc->priv->last_replace_text = g_strdup (replace);
+
+	gedit_document_end_user_action (doc);
+}
+
+gboolean
+gedit_document_replace_all (GeditDocument *doc,
+	      	const gchar *find, const gchar *replace, gboolean case_sensitive)
+{
+	gboolean from_cursor = FALSE;
+	gboolean cont = 0;
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+	g_return_val_if_fail (find != NULL, FALSE);
+	g_return_val_if_fail (replace != NULL, FALSE);
+
+	gedit_document_begin_user_action (doc);
+
+	while (gedit_document_find (doc, find, from_cursor, case_sensitive)) 
+	{
+		gedit_document_replace_selected_text (doc, replace);
+		
+		from_cursor = TRUE;
+		++cont;
+	}
+
+	gedit_document_end_user_action (doc);
+
+	return cont;
+}
+
