@@ -4,7 +4,7 @@
  * This file is part of gedit
  *
  * Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
- * Copyright (C) 2000, 2002 Chema Celorio, Paolo Maggi 
+ * Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA.
+ * Boston, MA 02111-1307, USA. * *
  */
  
 /*
- * Modified by the gedit Team, 1998-2002. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2001. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  */
@@ -59,6 +59,8 @@ struct _GeditDocumentClass
 {
 	GtkTextBufferClass parent_class;
 
+	GtkTextTagTable *tag_table;
+
 	/* File name (uri) changed */
 	void (* name_changed)		(GeditDocument *document);
 
@@ -78,10 +80,9 @@ struct _GeditDocumentClass
 					 gboolean can_redo);
 
 };
-#define GEDIT_ERROR_INVALID_UTF8_DATA 	1024
-#define GEDIT_ERROR_UNTITLED		1025	
-#define GEDIT_DOCUMENT_IO_ERROR gedit_document_io_error_quark ()
-GQuark 		gedit_document_io_error_quark (void);
+
+#define GEDIT_DOCUMENT_IO_ERROR gedit_document_io_error_quark()
+GQuark 		gedit_document_io_error_quark();
 
 GType        	gedit_document_get_type      	(void) G_GNUC_CONST;
 
@@ -91,7 +92,6 @@ GeditDocument* 	gedit_document_new_with_uri 	(const gchar *uri, GError **error);
 void 		gedit_document_set_readonly 	(GeditDocument *doc, gboolean readonly);
 gboolean	gedit_document_is_readonly 	(GeditDocument *doc);
 
-gchar*		gedit_document_get_raw_uri 	(const GeditDocument* doc);
 gchar*		gedit_document_get_uri 		(const GeditDocument* doc);
 gchar*		gedit_document_get_short_name 	(const GeditDocument* doc);
 
@@ -105,37 +105,29 @@ gboolean	gedit_document_save 		(GeditDocument* doc, GError **error);
 gboolean	gedit_document_save_as 		(GeditDocument* doc, 
 						 const gchar *uri, GError **error);
 
-gboolean	gedit_document_save_a_copy_as 	(GeditDocument* doc, 
-						 const gchar *uri, GError **error);
-
 gboolean 	gedit_document_revert 		(GeditDocument *doc,  GError **error);
 
 gboolean	gedit_document_is_untitled 	(const GeditDocument* doc);
 gboolean	gedit_document_get_modified 	(const GeditDocument* doc);
 
+gchar* 		gedit_document_get_buffer 	(const GeditDocument *doc);
 gint		gedit_document_get_char_count	(const GeditDocument *doc);
 gint		gedit_document_get_line_count 	(const GeditDocument *doc);
 		
+void 		gedit_document_delete_all_text 	(GeditDocument *doc);
+
 void		gedit_document_insert_text	(GeditDocument *doc, 
 						 gint	        pos,
                                                  const gchar   *text,
                                                  gint           len);
-
-void		gedit_document_insert_text_at_cursor (GeditDocument *doc, 
-                                                 const gchar   *text,
-                                                 gint           len);
-
-gint		gedit_document_get_cursor (GeditDocument *doc);
-void		gedit_document_set_cursor (GeditDocument *doc, gint cursor);
-
 void 		gedit_document_delete_text 	(GeditDocument *doc, 
 		                                 gint start, gint end);
 
 gchar*		gedit_document_get_chars 	(GeditDocument *doc, 
 					         gint start, gint end);
 
-gboolean	gedit_document_get_selection 	(GeditDocument *doc,
-						 gint* start, gint* end);
+gchar*		gedit_document_get_selected_text (GeditDocument *doc);
+gboolean	gedit_document_has_selected_text (GeditDocument *doc);
 
 /* Multi-level Undo/Redo operations */
 gboolean	gedit_document_can_undo		(const GeditDocument *doc);
@@ -155,9 +147,6 @@ void		gedit_document_goto_line 	(GeditDocument* doc, guint line);
 gchar* 		gedit_document_get_last_searched_text (GeditDocument* doc);
 gchar* 		gedit_document_get_last_replace_text  (GeditDocument* doc);
 
-void 		gedit_document_set_last_searched_text (GeditDocument* doc, const gchar *text);
-void 		gedit_document_set_last_replace_text  (GeditDocument* doc, const gchar *text);
-
 gboolean	gedit_document_find 		(GeditDocument* doc, const gchar* str, 
 						 gboolean from_cursor, 
 						 gboolean case_sensitive);
@@ -165,15 +154,9 @@ gboolean	gedit_document_find_again	(GeditDocument* doc);
 
 void		gedit_document_replace_selected_text (GeditDocument *doc, 
 						      const gchar *replace);
-gboolean	gedit_document_replace_all 	(GeditDocument *doc,
-				            	 const gchar *find, 
-						 const gchar *replace, 
-					    	 gboolean case_sensitive);
-guint		gedit_document_get_line_at_offset (const GeditDocument *doc, guint offset);
-
-void		gedit_document_set_selection 	(GeditDocument *doc, 
-						 gint start, 
-						 gint end);
+gboolean	gedit_document_replace_all (GeditDocument *doc,
+				            const gchar *find, const gchar *replace, 
+					    gboolean case_sensitive);
 
 #endif /* __GEDIT_DOCUMENT_H__ */
 
