@@ -5,7 +5,6 @@
  *
  * Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
  * Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi 
- * Copyright (C) 2002  Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +19,11 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA. 
+ * Boston, MA 02111-1307, USA. * *
  */
  
 /*
- * Modified by the gedit Team, 1998-2002. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2001. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  */
@@ -33,8 +32,6 @@
 #define __GEDIT_MDI_H__
 
 #include <bonobo-mdi.h>
-#include "gedit-recent.h"
-#include "recent-files/egg-recent-view.h"
 
 #define GEDIT_TYPE_MDI			(gedit_mdi_get_type ())
 #define GEDIT_MDI(obj)			(GTK_CHECK_CAST ((obj), GEDIT_TYPE_MDI, GeditMDI))
@@ -68,12 +65,10 @@ GeditMDI*	gedit_mdi_new		(void);
 
 void		gedit_mdi_set_active_window_title (BonoboMDI *mdi);
 
+void		gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi);
+
 /* FIXME: should be static ??? */
 void 		gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi);
 
-void 		gedit_mdi_clear_active_window_statusbar (GeditMDI *mdi);
-
-EggRecentView	*gedit_mdi_get_recent_view_from_window (BonoboWindow* win);
-
 #endif /* __GEDIT_MDI_H__ */
 
