@@ -3,8 +3,7 @@
  * gedit-print.c
  * This file is part of gedit
  *
- * Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi
- * Copyright (C) 2002  Paolo Maggi  
+ * Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,21 +22,16 @@
  */
  
 /*
- * Modified by the gedit Team, 1998-2002. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2001. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <libgnome/libgnome.h>
 #include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprint/gnome-print-master.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
-#include <eel/eel-string.h>
+#include <libgnomeprintui/gnome-print-master-preview.h>
 
 #include <string.h>	/* For strlen */
 
@@ -45,15 +39,12 @@
 #include "gedit-print.h"
 #include "gedit-debug.h"
 #include "gedit-document.h"
-#include "gedit-prefs-manager.h"
+#include "gedit-prefs.h"
 
 #define CM(v) ((v) * 72.0 / 2.54)
 #define A4_WIDTH (210.0 * 72 / 25.4)
 #define A4_HEIGHT (297.0 * 72 / 25.4)
 
-
-static GnomePrintConfig *gedit_print_config = NULL;
-
 typedef struct _GeditPrintJobInfo	GeditPrintJobInfo;
 
 struct _GeditPrintJobInfo {
@@ -65,7 +56,7 @@ struct _GeditPrintJobInfo {
 	GnomePrintConfig 	*config;
 	gint			 range_type;
 
-	GnomePrintJob		*print_job;
+	GnomePrintMaster	*print_master;
 	GnomePrintContext	*print_ctx;
 
 	gint			 page_num;
@@ -86,15 +77,9 @@ struct _GeditPrintJobInfo {
 
 	gint 			 first_line_to_print;
 	gint			 printed_lines;
-
-	gint 			 print_line_numbers;
-	gboolean		 print_header;
-	gint			 tabs_size;
-	GtkWrapMode		 print_wrap_mode;
 };
 
-
-static GQuark gedit_print_error_quark (void);
+static GQuark gedit_print_error_quark ();
 #define GEDIT_PRINT_ERROR gedit_print_error_quark ()	
 
 static GeditPrintJobInfo* gedit_print_job_info_new (GeditDocument* doc, GError **error);
@@ -135,22 +120,10 @@ gedit_print_job_info_new (GeditDocument* doc, GError **error)
 {	
 	GeditPrintJobInfo *pji;
 	
-	gchar *print_font_body;
-	gchar *print_font_header;
-	gchar *print_font_numbers;
-	
 	gedit_debug (DEBUG_PRINT, "");
 	
 	g_return_val_if_fail (doc != NULL, NULL);
-
-	if (gedit_print_config == NULL)
-	{
-		gedit_print_config = gnome_print_config_default ();
-		g_return_val_if_fail (gedit_print_config != NULL, NULL);
-
-		gnome_print_config_set (gedit_print_config, "Settings.Transport.Backend", "lpr");
-		gnome_print_config_set (gedit_print_config, "Printer", "GENERIC");
-	}
+	g_return_val_if_fail (gedit_settings != NULL, NULL);
 
 	pji = g_new0 (GeditPrintJobInfo, 1);
 
@@ -168,17 +141,14 @@ gedit_print_job_info_new (GeditDocument* doc, GError **error)
 	pji->margin_bottom = CM (1);
 	pji->header_height = CM (0);
 
-	print_font_body = gedit_prefs_manager_get_print_font_body ();
-	print_font_header = gedit_prefs_manager_get_print_font_header ();
-	print_font_numbers = gedit_prefs_manager_get_print_font_numbers ();
-	
-	pji->font_body = gnome_font_find_closest_from_full_name (print_font_body);
-	pji->font_header = gnome_font_find_closest_from_full_name (print_font_header);
-	pji->font_numbers = gnome_font_find_closest_from_full_name (print_font_numbers);
+	pji->font_body = gnome_font_find_closest_from_full_name (
+					gedit_settings->print_font_body);
 
-	g_free (print_font_body);
-	g_free (print_font_header);
-	g_free (print_font_numbers);
+	pji->font_header = gnome_font_find_closest_from_full_name (
+					gedit_settings->print_font_header);
+
+	pji->font_numbers = gnome_font_find_closest_from_full_name (
+					gedit_settings->print_font_numbers);
 
 	if ((pji->font_body == NULL) || 
 	    (pji->font_header == NULL) ||
@@ -192,8 +162,8 @@ gedit_print_job_info_new (GeditDocument* doc, GError **error)
 		
 		return NULL;
 	}
-	
-	if (gedit_prefs_manager_get_print_header ())
+
+	if (gedit_settings->print_header)
 		pji->header_height = 2.5 * gnome_font_get_size (pji->font_header);
 	else
 		pji->header_height = 0;
@@ -203,14 +173,6 @@ gedit_print_job_info_new (GeditDocument* doc, GError **error)
 	pji->first_line_to_print = 1;
 	pji->printed_lines = 0;
 
-	pji->print_line_numbers = gedit_prefs_manager_get_print_line_numbers ();
-	pji->print_header = gedit_prefs_manager_get_print_header ();
-	pji->tabs_size = gedit_prefs_manager_get_tabs_size ();
-	pji->print_wrap_mode = gedit_prefs_manager_get_print_wrap_mode ();
-		
-	pji->config = gedit_print_config;
-	gnome_print_config_ref (pji->config);
-
 	return pji;
 }	
 
@@ -227,8 +189,8 @@ gedit_print_job_info_destroy (GeditPrintJobInfo *pji)
 	if (pji->print_ctx != NULL)
 		g_object_unref (pji->print_ctx);
 
-	if (pji->print_job != NULL)
-		g_object_unref (pji->print_job);
+	if (pji->print_master != NULL)
+		g_object_unref (pji->print_master);
 
 	g_free (pji);
 }
@@ -240,7 +202,7 @@ gedit_print_update_page_size_and_margins (GeditPrintJobInfo *pji)
 		
 	gedit_debug (DEBUG_PRINT, "");
 
-	gnome_print_job_get_page_size_from_config (pji->config, 
+	gnome_print_master_get_page_size_from_config (pji->config, 
 			&pji->page_width, &pji->page_height);
 
 	if (gnome_print_config_get_length (pji->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, 
@@ -266,7 +228,7 @@ gedit_print_update_page_size_and_margins (GeditPrintJobInfo *pji)
 		gnome_print_convert_distance (&pji->margin_bottom, unit, GNOME_PRINT_PS_UNIT);
 	}
 
-	if (pji->print_line_numbers  > 0)
+	if (gedit_settings->print_line_numbers > 0)
 	{
 		gchar* num_str;
 
@@ -295,16 +257,13 @@ gedit_print_run_dialog (GeditPrintJobInfo *pji)
 
 	g_return_val_if_fail (pji != NULL, TRUE);
 	
-	if (!gedit_document_get_selection (pji->doc, NULL, NULL))
+	if (!gedit_document_has_selected_text (pji->doc))
 		selection_flag = GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE;
 	else
 		selection_flag = GNOME_PRINT_RANGE_SELECTION;
 		
-	dialog = g_object_new (GNOME_TYPE_PRINT_DIALOG, "print_config", pji->config, NULL);
-	
-	gnome_print_dialog_construct (GNOME_PRINT_DIALOG (dialog), 
-				      _("Print"),
-			              GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
+	dialog = gnome_print_dialog_new ((const char *) _("gedit - Print Document"),
+			          GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
 	
 	lines = gedit_document_get_line_count (pji->doc);
 	
@@ -335,6 +294,9 @@ gedit_print_run_dialog (GeditPrintJobInfo *pji)
 			return TRUE;
 	}
 	
+	g_return_val_if_fail (pji->config == NULL, TRUE);
+	pji->config = gnome_print_dialog_get_config (GNOME_PRINT_DIALOG (dialog));
+
 	pji->range_type = gnome_print_dialog_get_range (GNOME_PRINT_DIALOG (dialog));
 
 	gtk_widget_destroy (dialog);
@@ -350,9 +312,7 @@ gedit_print_header (GeditPrintJobInfo *pji, gint page_number)
 	gdouble y, x, len;
 	gchar* l_text;
 	gchar* r_text;
-	gchar* uri;
-	gchar* uri_to_print;
-	
+
 	gedit_debug (DEBUG_PRINT, "");	
 
 	g_return_if_fail (pji != NULL);
@@ -367,13 +327,7 @@ gedit_print_header (GeditPrintJobInfo *pji, gint page_number)
 	/* Print left text */
 	x = pji->margin_right;
 	
-	uri = gedit_document_get_uri (pji->doc);
-	uri_to_print = eel_str_middle_truncate (uri, 60);
-	g_free (uri);
-
-	l_text = g_strdup_printf (_("File: %s"), uri_to_print);
-	g_free (uri_to_print);
-
+	l_text = g_strdup_printf (_("File: %s"), gedit_document_get_uri (pji->doc));
 	gnome_print_moveto (pji->print_ctx, x, y);
 	gnome_print_show (pji->print_ctx, l_text);
 
@@ -413,30 +367,18 @@ gedit_print_document (GeditPrintJobInfo *pji)
 	gint paragraph_delimiter_index;
 	gint next_paragraph_start;
 	gdouble fontheight;
-	gint start, end_sel;
-	
+
 	gedit_debug (DEBUG_PRINT, "");	
 	
 	switch (pji->range_type)
 	{
 		case GNOME_PRINT_RANGE_ALL:
-			text_to_print = gedit_document_get_chars (pji->doc, 0, -1);
+			text_to_print = gedit_document_get_buffer (pji->doc);
 			text_end = text_to_print + strlen (text_to_print);
 			break;
 		case GNOME_PRINT_RANGE_SELECTION:
-			if (gedit_document_get_selection (pji->doc, &start, &end_sel))
-			{
-				text_to_print = gedit_document_get_chars (pji->doc, 
-							start, end_sel);
-				
-				text_end = text_to_print + strlen (text_to_print);
-		
-				pji->first_line_to_print =  
-					gedit_document_get_line_at_offset (pji->doc, start) + 1;
-			}
-			else
-				g_return_if_fail (FALSE);
-
+			text_to_print = gedit_document_get_selected_text (pji->doc);
+			text_end = text_to_print + strlen (text_to_print);
 			break;
 		default:
 			g_return_if_fail (FALSE);
@@ -517,10 +459,10 @@ gedit_print_preview_real (GeditPrintJobInfo *pji)
 	gedit_debug (DEBUG_PRINT, "");
 
 	g_return_if_fail (pji != NULL);
-	g_return_if_fail (pji->print_job != NULL);
+	g_return_if_fail (pji->print_master != NULL);
 
 	title = g_strdup_printf (_("gedit - Print Preview"));
-	gpmp = gnome_print_job_preview_new (pji->print_job, title);
+	gpmp = gnome_print_master_preview_new (pji->print_master, title);
 	g_free (title);
 
 	gtk_widget_show (gpmp);
@@ -553,6 +495,8 @@ gedit_print_real (GeditDocument* doc, gboolean preview, GError **error)
 		 * pji->config 
 		 */
 		cancel = gedit_print_run_dialog (pji);
+	else
+		pji->config = gnome_print_config_default ();
 
 	/* The canceled button on the dialog was clicked */
 	if (cancel) {
@@ -562,10 +506,10 @@ gedit_print_real (GeditDocument* doc, gboolean preview, GError **error)
 
 	g_return_if_fail (pji->config != NULL);
 
-	pji->print_job = gnome_print_job_new (pji->config);
-	g_return_if_fail (pji->print_job != NULL);
+	pji->print_master = gnome_print_master_new_from_config (pji->config);
+	g_return_if_fail (pji->print_master != NULL);
 
-	pji->print_ctx = gnome_print_job_get_context (pji->print_job);
+	pji->print_ctx = gnome_print_master_get_context (pji->print_master);
 	g_return_if_fail (pji->print_ctx != NULL);
 
 	gedit_print_update_page_size_and_margins (pji);
@@ -577,12 +521,12 @@ gedit_print_real (GeditDocument* doc, gboolean preview, GError **error)
 		return;
 	}
 #endif	
-	gnome_print_job_close (pji->print_job);
+	gnome_print_master_close (pji->print_master);
 
 	if (pji->preview)
 		gedit_print_preview_real (pji);
 	else
-		gnome_print_job_print (pji->print_job);
+		gnome_print_master_print (pji->print_master);
 	
 	gedit_print_job_info_destroy (pji);
 }
@@ -591,15 +535,17 @@ static void
 gedit_print_error_dialog (GError *error)
 {
 	GtkWidget *dialog;
-
+	gchar *errstr;
+		       	
+	errstr = g_strdup_printf (_("An error occurred while printing.\n\n%s,"), error->message);
+		
 	dialog = gtk_message_dialog_new (
 			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
 			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 		   	GTK_MESSAGE_ERROR,
 		   	GTK_BUTTONS_OK,
-			_("An error occurred while printing.\n\n%s,"),
-			error->message);
-
+			errstr);
+			
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
@@ -607,6 +553,8 @@ gedit_print_error_dialog (GError *error)
 	gtk_dialog_run (GTK_DIALOG (dialog));
 
 	gtk_widget_destroy (dialog);
+			
+	g_free (errstr);
 }
 
 
@@ -659,12 +607,13 @@ gedit_print_get_next_line_to_print_delimiter (GeditPrintJobInfo *pji,
 		const gchar *start, const gchar *end, gboolean first_line_of_par)
 {
 	const gchar* p;
-	const gchar* t;
 	gdouble line_width = 0.0;
 	gdouble printable_page_width;
 	ArtPoint space_advance;
 	gint space;
+
 	gint chars_in_this_line = 0;
+	gint tab_size = gedit_settings->tab_size;
 	
 	gedit_debug (DEBUG_PRINT, "");
 
@@ -676,7 +625,6 @@ gedit_print_get_next_line_to_print_delimiter (GeditPrintJobInfo *pji,
 			(pji->margin_right + pji->margin_left);
 	
 	p = start;
-	t = start;
 
 	/* Find space advance */
 	space = gnome_font_lookup_default (pji->font_body, ' ');
@@ -706,21 +654,18 @@ gedit_print_get_next_line_to_print_delimiter (GeditPrintJobInfo *pji,
 			/* FIXME: use a tabs array */
 			gint num_of_equivalent_spaces;
 			
-			num_of_equivalent_spaces = pji->tabs_size - ((chars_in_this_line - 1) % pji->tabs_size); 
+			num_of_equivalent_spaces = tab_size - ((chars_in_this_line - 1) % tab_size); 
 			chars_in_this_line += num_of_equivalent_spaces - 1;
 			
 			line_width += (num_of_equivalent_spaces * space_advance.x); 
-			t = p;
 		}
 		else
 		{
 			glyph = gnome_font_lookup_default (pji->font_body, ch);
 
 			/* FIXME */
-			if (glyph == space){			
+			if (glyph == space)
 				line_width += space_advance.x;
-				t = p;
-			}
 			else
 			/*
 			if ((glyph < 0) || (glyph >= 256))
@@ -744,15 +689,8 @@ gedit_print_get_next_line_to_print_delimiter (GeditPrintJobInfo *pji,
 
 		if (line_width > printable_page_width)
 		{
-                        /* Fix for word wrap at line boundaries */
-                        if (pji->print_wrap_mode == GTK_WRAP_WORD)
-			{
-                                if (t == start)   /* handling words longer than a line */
-                                        return p;
-                                return t;	 
-                        }
-
-                        return p; /* keeping char-wrap and wrap-none unaltered */
+			/* FIXME: take care of lines wrapping at word boundaries too */
+			return p;
 		}
 
 		p = g_utf8_next_char (p);
@@ -770,9 +708,7 @@ gedit_print_line_number (GeditPrintJobInfo *pji, gdouble y)
 		
 	gint line_num = pji->first_line_to_print + pji->printed_lines;
 
-	gedit_debug (DEBUG_PRINT, "");
-
-	if ((pji->printed_lines + 1) % pji->print_line_numbers != 0)
+	if ((pji->printed_lines + 1) % gedit_settings->print_line_numbers != 0)
 	       return;	
 		
 	num_str = g_strdup_printf ("%d", line_num);
@@ -801,6 +737,7 @@ gedit_print_line (GeditPrintJobInfo *pji, const gchar *start, const gchar *end,
 	gdouble x;
 	const gchar* p;
 	gint chars_in_this_line = 0;
+	gint tab_size = gedit_settings->tab_size;
 	
 	gedit_debug (DEBUG_PRINT, "");
 
@@ -849,9 +786,7 @@ gedit_print_line (GeditPrintJobInfo *pji, const gchar *start, const gchar *end,
 			
 			glyph = gnome_font_lookup_default (pji->font_body, ' ');
 			
-			num_of_equivalent_spaces = pji->tabs_size - 
-				((chars_in_this_line - 1) % pji->tabs_size); 
-
+			num_of_equivalent_spaces = tab_size - ((chars_in_this_line - 1) % tab_size); 
 			chars_in_this_line += num_of_equivalent_spaces - 1;
 			
 			for (i = 0; i < num_of_equivalent_spaces; ++i)
@@ -903,8 +838,7 @@ gedit_print_paragraph (GeditPrintJobInfo *pji, const gchar *start, const gchar *
 			y = gedit_print_create_new_page (pji);
 		}
 
-		/* FIXME: "word wrap" - Paolo */
-		if (pji->print_wrap_mode == GTK_WRAP_NONE)
+		if (!gedit_settings->print_wrap_lines)
 			return y;
 
 	}
@@ -917,15 +851,13 @@ gedit_print_create_new_page (GeditPrintJobInfo *pji)
 {
 	gchar *pn_str;
 
-	gedit_debug (DEBUG_PRINT, "");
-
 	++pji->page_num;
 	pn_str = g_strdup_printf ("%d", pji->page_num);
 	gnome_print_beginpage (pji->print_ctx, pn_str);
 
 	g_free (pn_str);
 
-	if (pji->print_header)
+	if (gedit_settings->print_header)
 	{	
 		gedit_print_header (pji, +pji->page_num);
 		return pji->page_height - pji->margin_top - pji->header_height;
